﻿using BlacksmithWorkshopContracts.BindingModels;
using BlacksmithWorkshopContracts.SearchModels;
using BlacksmithWorkshopContracts.StoragesContracts;
using BlacksmithWorkshopContracts.ViewModels;
using BlacksmithWorkshopListImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BlacksmithWorkshopListImplement.Implements
{
    //класс, реализующий интерфейс хранилища изделий
    public class ArticleStorage : IArticleStorage
    {
        //поле для работы со списком изделий
        private readonly DataListSingleton _source;

        //получение в конструкторе объекта DataListSingleton
        public ArticleStorage()
        {
            _source = DataListSingleton.GetInstance();
        }

        //получение полного списка изделий
        public List<ArticleViewModel> GetFullList()
        {
            var result = new List<ArticleViewModel>();

            foreach(var article in _source.Articles)
            {
                result.Add(article.GetViewModel);
            }

            return result;
        }

        //получение отфильтрованного списка изделий
        public List<ArticleViewModel> GetFilteredList(ArticleSearchModel model)
        {
            var result = new List<ArticleViewModel>();

            if (string.IsNullOrEmpty(model.ArticleName))
            {
                return result;
            }

            foreach(var article in _source.Articles)
            {
                if(article.ArticleName.Contains(model.ArticleName))
                {
                    result.Add(article.GetViewModel);
                }
            }

            return result;
        }

        //получение элемента из списка изделий
        public ArticleViewModel? GetElement(ArticleSearchModel model)
        {
            if(string.IsNullOrEmpty(model.ArticleName) && !model.Id.HasValue)
            {
                return null;
            }

            foreach(var article in _source.Articles)
            {
                if((!string.IsNullOrEmpty(model.ArticleName) && article.ArticleName == model.ArticleName) ||
                    (model.Id.HasValue && article.Id == model.Id))
                {
                    return article.GetViewModel;
                }
            }

            return null;
        }

        //при создании изделия определяем для него новый id: ищем max id и прибавлляем к нему 1
        public ArticleViewModel? Insert(ArticleBindingModel model)
        {
            model.Id = 1;

            foreach(var article in _source.Articles)
            {
                if(model.Id <= article.Id)
                {
                    model.Id = article.Id + 1;
                }
            }

            var newArticle = Article.Create(model);

            if(newArticle == null)
            {
                return null;
            }

            _source.Articles.Add(newArticle);

            return newArticle.GetViewModel;
        }

        //обновление изделия
        public ArticleViewModel? Update(ArticleBindingModel model)
        {
            foreach (var article in _source.Articles)
            {
                if (article.Id == model.Id)
                {
                    article.Update(model);

                    return article.GetViewModel;
                }
            }

            return null;
        }

        //удаление изделия
        public ArticleViewModel? Delete(ArticleBindingModel model)
        {
            for(int i = 0; i < _source.Articles.Count; ++i)
            {
                if (_source.Articles[i].Id == model.Id)
                {
                    var element = _source.Articles[i];
                    _source.Articles.RemoveAt(i);

                    return element.GetViewModel;
                }
            }

            return null;
        }
    }
}
